package RPM::Source::BundleImport::Metadata;

use strict;
use warnings;
use Carp;

use RPM::Source::Abstract::PriorityCollection;

our $VERSION=0.005;
our $verbose=0;
our $collect_all_metadata=0;

use Exporter 'import';
our @EXPORT = qw(
MPRIORITY_OVERRIDE
MPRIORITY_PARENT_OVERRIDE
MPRIORITY_META_OFFICIAL
MPRIORITY_EVAL
MPRIORITY_DEFAULT
MPRIORITY_GUESS
MPRIORITY_OLD_PARENT
MPRIORITY_FALLBACK
MPRIORITY_DISCARD
);

use constant {
	MPRIORITY_OVERRIDE	=> 80,
	MPRIORITY_PARENT_OVERRIDE=>70,
	MPRIORITY_META_OFFICIAL	=> 60,
	MPRIORITY_EVAL		=> 50,
	MPRIORITY_DEFAULT	=> 40,
	MPRIORITY_GUESS		=> 30,
	MPRIORITY_OLD_PARENT	=> 20,
	MPRIORITY_FALLBACK	=> 10,
	MPRIORITY_DISCARD	=> 0,
};

our %metadata_aliases=qw/
abstract summary
shortdesc summary
title summary
homepage url
/;

# note: 'docfiles' is used in texlive tlpobj
my @metadata_keys=qw/
name
version
summary
description
license
url
rpmgroup

docfiles:list
/;

my @_KNOWN_KEYS_add=qw/
_metadata_name
_metadata_version
/;
my %_KNOWN_KEYS= map {$_=>1} @_KNOWN_KEYS_add, @metadata_keys;

my %validators = (
    'summary' => \&__has_an_ascii_letter_and_not_unknown,
    'description' => \&__has_an_ascii_letter_and_not_unknown,
    'license' => \&__has_an_ascii_letter_and_not_unknown,
    'url' => \&__has_an_ascii_letter_and_not_unknown,
    'name' => \&__non_space,
    'version' => \&__non_space,
    );

my %trimmers = (
    'summary' => \&__trim_space,
    'license' => \&__trim_space,
    'url' => \&__trim_space,
    'name' => \&__trim_space,
    'version' => \&__trim_space,
    );

my %evaluators = (
    'summary' => \&__has_at_least_two_words,
    );

sub __has_an_ascii_letter_and_not_unknown {
    my $in=shift;
    return ($in ne 'UNKNOWN') && ($in ne 'unknown') && ($in=~/[a-zA-Z]/);
}
sub __has_at_least_two_words {
    return $_[0]=~/[a-zA-Z]\s+[a-zA-Z]/;
}
sub __non_space {
    return $_[0]=~/\S/;
}

sub __trim_space {
    my ($data)=@_;
    if ($data) {
	$data=~s/^\s*//;
	$data=~s/\s*$//;
    }
    return $data;
}


sub new {
    my $class = shift;
    my $self = {
	-metadata => {},
	@_
    };
    bless $self, $class;
    return $self;
}

sub add {
    my $self=shift;
    my $key=shift;
    croak "Metadata: add: odd number in hash assignment" if (scalar @_)%2 == 1;
    my $datahash={
	-priority => MPRIORITY_DISCARD,
	@_};
    my $val=$datahash->{-value};
    return if !defined($val);

    $key=&__key_subst($key);
    if (!ref ($val)) {
	my $validator=$validators{$key};
	return if $validator && ! &$validator($val);
	my $trimmer=$trimmers{$key};
	$val=$datahash->{-value}=&$trimmer($val) if $trimmer;
	my $evaluator=$evaluators{$key};
	$datahash->{-priority} = MPRIORITY_DISCARD if $evaluator && ! &$evaluator($val);
    }
    my $storobj=$self->{-metadata}->{$key};
    unless ($storobj) {
	$storobj=RPM::Source::Abstract::PriorityCollection->new();
	$self->{-metadata}->{$key}=$storobj;
    }
    print STDERR "Metadata: added $key: priority=",$datahash->{-priority}," value=",$datahash->{-value},
    ($datahash->{-description}?' description=['.$datahash->{-description}.']':''),"\n" if $verbose;
    $storobj->add($datahash);
}

sub has {
    my $self=shift;
    my $key=shift;
    $key=&__key_subst($key);
    my $storobj=$self->{-metadata}->{$key};
    return 1 if $storobj;
    return 0;
}

sub want {
    my ($self,$key,$priority)=@_;
    return 1 if $collect_all_metadata;
    $key=&__key_subst($key);
    my $storobj=$self->{-metadata}->{$key};
    return 1 unless $storobj;
    return 1 if $storobj->[0]->{-priority} < $priority;
    return 0;
}

sub getfirst {
    my $self=shift;
    my $key=shift;
    $key=&__key_subst($key);
    my $aref=$self->{-metadata}->{$key};
    return if ! $aref;
    return $aref->[0]->{-value};
}

sub __key_subst {
    my $key=shift;
    my $keysubst=$metadata_aliases{$key};
    $key=$keysubst if $keysubst;
    unless ($_KNOWN_KEYS{$key}) {
	Carp::confess "Oops! internal error: unknown key $key";
    }
    return $key;
}

sub collect_metadata_hash {
    my $self=shift;
    my $metadata=shift;
    my %opt=(
	-priority => MPRIORITY_META_OFFICIAL,
	-description => 'collect_metadata_hash',
	@_
	);
    my @shared_args=(-priority => $opt{-priority}, -description => $opt{-description});
    my %collision;
    #foreach my $key (@metadata_keys, keys (%metadata_aliases)) {
    foreach my $raw_key (keys (%$metadata)) {
	my $value = $metadata->{$raw_key};
	my $key=lc($raw_key);
	my $unaliased_key=$metadata_aliases{$key};
	$key=$unaliased_key if $unaliased_key;
	if ($_KNOWN_KEYS{$key} and defined($value)) {
	    #print STDERR "DEBUG: collect_metadata_hash: known key $key($raw_key): ",(defined($value)?$value:'undef'),"\n";
	    if ($collision{$key}) {
		warn "warn: collect_metadata_hash: possible collision for $raw_key\n";
	    }
	    $collision{$key}=1;
	    $collision{$unaliased_key}=1 if $unaliased_key;
	    $self->add($key, @shared_args, -value => $value);
	    if ($_KNOWN_KEYS{'_metadata_'.$key}) {
		$self->add('_metadata_'.$key, @shared_args, -value => $value);
	    }
	#} elsif (not $_KNOWN_KEYS{$key}) {
	#    print STDERR "DEBUG: collect_metadata_hash: unknown key $key($raw_key): ",(defined($value)?$value:'undef'),"\n";
	}
    }
    my $resources=$metadata->{'resources'};
    if ($resources) {
	foreach my $key ('homepage') {
	    $self->add($key, @shared_args,
	       -value => $resources->{$key});
	}
    }
}

1;
